!-----------------------------------------------------------------------------
! gnuplot_data_generator.f90
!  
!    Createdy by:  Akira Kageyama, kage@jamstec.go.jp, 
!                  Earth Simulator Center,
!                  JAMSTEC, Japan
!             on:  2007.07.12
!            for:  Les Houches Summer School "Dynamos". The kindanb code.
!-----------------------------------------------------------------------------

program main
  use constants
  use ut
  use namelist
  use grid
  use turtle
  implicit none

  ! - 2-D single precision real arrays.
  real(SP), dimension(:,:), allocatable :: Slice_vx  ! x-comp. of velocity
  real(SP), dimension(:,:), allocatable :: Slice_vy  ! y-comp.
  real(SP), dimension(:,:), allocatable :: Slice_vz  ! z-comp.
  real(SP), dimension(:,:), allocatable :: Slice_bx  ! x-comp. of mag. field
  real(SP), dimension(:,:), allocatable :: Slice_by  ! y-comp.
  real(SP), dimension(:,:), allocatable :: Slice_bz  ! z-comp.  

  character(len=TAG_STRING_LENGTH_MAX) :: datafilename

  integer :: target_nloop

  type(turtle__pos_) :: window_lower_left, window_upper_right
  type(turtle__pos_) :: shift

  type contour_info_
     integer  :: nlevels
     real(SP) :: vmin
     real(SP) :: vmax
  end type contour_info_

  call namelist__read
  call grid__initialize

  allocate(Slice_vx(NX,NY),   &
           Slice_vy(NX,NY),   &
           Slice_vz(NX,NY))

  allocate(Slice_bx(NX,NY),   &
           Slice_by(NX,NY),   &
           Slice_bz(NX,NY))

  window_lower_left%x  = -2.0
  window_lower_left%y  = -2.0
  window_upper_right%x =  2.0
  window_upper_right%y =  2.0

  call turtle__initialize(window_lower_left, window_upper_right)

!  shift%x = 0.0 
!  shift%y = 0.0
!  call turtle__coords_shift(shift)

  call read_slice_data

  call draw_zplane

  call draw_boundary_box


contains


!________________________________________________________________________
!                                                                        !
  subroutine draw_boundary_box                                           !
!________________________________________________________________________!
!
    type(turtle__pos_) :: corner_southwest, corner_northeast

    corner_southwest%x = grid__pos%x(1)
    corner_southwest%y = grid__pos%y(1)

print *,'  x,y = ', grid__pos%x(1), grid__pos%y(1)

    corner_northeast%x = grid__pos%x(NX)
    corner_northeast%y = grid__pos%y(NY)

print *,'  x,y = ', grid__pos%x(NX), grid__pos%y(NY)

    open(FILE_FOR_TURTLE,                                               &
         file=trim(turtle__filename_for_lines("boundary_box")))

    call turtle__rectangle(corner_southwest,corner_northeast)

    close(FILE_FOR_TURTLE)

  end subroutine draw_boundary_box


!________________________________________________________________________
!                                                                        !
  subroutine draw_zplane                                                 !
!________________________________________________________________________!
!

!==================
! Velocity field     
!==================
    open(FILE_FOR_TURTLE,                              &
         file=trim(turtle__filename_for_lines("zslice_contour_vz")))
    call draw_zplane_contour(Slice_vz)
    close(FILE_FOR_TURTLE)

    open(FILE_FOR_TURTLE,                              &
         file=trim(turtle__filename_for_lines("zslice_vector_vel")))
    call draw_zplane_vector(Slice_vx,                  &
                            Slice_vy)
    close(FILE_FOR_TURTLE)

!==================
! Magnetic field     
!==================
    open(FILE_FOR_TURTLE,                              &
         file=trim(turtle__filename_for_lines("zslice_contour_bz")))
    call draw_zplane_contour(Slice_bz)
    close(FILE_FOR_TURTLE)

    open(FILE_FOR_TURTLE,                              &
         file=trim(turtle__filename_for_lines("zslice_vector_mag")))
    call draw_zplane_vector(Slice_bx,                  &
                            Slice_by)
    close(FILE_FOR_TURTLE)

  end subroutine draw_zplane


!________________________________________________________________________
!                                                                        !
  subroutine draw_zplane_contour(field)                                  !
    real(SP), intent(in), dimension(:,:) :: field                        !
!________________________________________________________________________!
!
    type(turtle__scalar2d_cartesian_) :: work
    integer :: contour_levels

    contour_levels = 20

    allocate(work%f(NX,NY),work%xpos(NX),work%ypos(NY))

    work%nx = NX
    work%ny = NY

    work%xpos(:) = grid__pos%x(:)
    work%ypos(:) = grid__pos%y(:)
    work%f(:,:)  = field(:,:)

    call turtle__contour_cartesian(work,contour_levels)

    deallocate(work%f,work%xpos,work%ypos)

  end subroutine draw_zplane_contour


!_______________________________________________________________private__
!                                                                        !
  subroutine draw_zplane_vector(vector_x,vector_y)                       !
    real(SP), dimension(:,:), intent(in) :: vector_x, vector_y           !
!________________________________________________________________________!
!
    type(turtle__vector2d_cartesian_) :: vec

    allocate(vec%x(NX,NY))
    allocate(vec%y(NX,NY))
    allocate(vec%xpos(NX))
    allocate(vec%ypos(NY))

    vec%nx = NX
    vec%ny = NY

    vec%xpos(:) = grid__pos%x(:)
    vec%ypos(:) = grid__pos%y(:)

    vec%x(:,:)  = vector_x(:,:)
    vec%y(:,:)  = vector_y(:,:)
    
!    call turtle__vector_cartesian(vec,norm=arrow_norm)
    call turtle__vector_cartesian(vec)

    deallocate(vec%x,vec%y,vec%xpos,vec%ypos)

  end subroutine draw_zplane_vector


!________________________________________________________________________
!                                                                        !
  subroutine read_slice_data                                             !
!________________________________________________________________________!
!
    integer  :: dummy_nloop
    real(SP) :: dummy_time

    character(len=TAG_STRING_LENGTH_MAX) :: filename
    integer :: target_nloop

    filename = namelist__string('Slicedata_filename')
    target_nloop = namelist__integer('Target_nloop')

    !-- To save nloop information on a file. This will be used
    !-- in the gnuplot script and labeled in the finale figure.
    open(FILE_FOR_TURTLE,file='Workfiles/.shot.nloop')
    write(FILE_FOR_TURTLE,*) Target_nloop
    close(FILE_FOR_TURTLE)

    print *,'Opening ', trim(filename)

    open(FILE_SLICEDATA,                &
         file=trim(filename),           &
         form='unformatted',            &
         status='old')

    do
       read(FILE_SLICEDATA) dummy_nloop, dummy_time,            &
                            Slice_vx, Slice_vy, Slice_vz,       &
                            Slice_bx, Slice_by, Slice_bz
       print *, ' dummy_nloop = ', dummy_nloop
       if (dummy_nloop==target_nloop) exit
    end do

    if (dummy_nloop/=target_nloop)                              &
         call ut__fatal('Could not find the target data.')

    close(FILE_SLICEDATA)

  end subroutine read_slice_data

end program main




