!-----------------------------------------------------------------------------
! KINDANB: kinematic dynamo in a box
!-----------------------------------------------------------------------------
! namelist.f90
!              2006.09.19: Created by Akira Kageyama.
!-----------------------------------------------------------------------------
module namelist
!*****************************************************************************
! MODULE NAMELIST                                              Namelist Loader
!*****************************************************************************
  use constants
  use ut
  implicit none
  private
  public :: & !< routines >!
            namelist__double,   &
            namelist__integer,  &
            namelist__logical,  &
            namelist__read,     &
            namelist__string

  logical, save :: Read_done = .false.

  integer                              :: Total_nloop
  integer                              :: Avs_nskip, Slicedata_nskip
  character(len=TAG_STRING_LENGTH_MAX) :: Avs_tag, Slicedata_tag
  real(DP)                             :: Magnetic_reynolds
  logical                              :: Debug, Verbose

  namelist /data00/ Total_nloop
  namelist /data01/ Avs_nskip,        Avs_tag
  namelist /data02/ Slicedata_nskip,  Slicedata_tag
  namelist /data03/ Magnetic_reynolds
  namelist /data04/ Debug, Verbose


contains


!________________________________________________________________public__
!                                                                        !
  function namelist__double(variable)                                    !
    character(len=*), intent(in) :: variable                             !
    real(DP) :: namelist__double                                         !
!________________________________________________________________________!
!
    if ( .not.Read_done ) then
       call ut__fatal('<namelist__double> Read namelist file first.')
    end if

    select case (variable)
    case                ('Magnetic_reynolds')
       namelist__double = Magnetic_reynolds
    case default
       call ut__message('? arg = ', variable)
       call ut__fatal('<namelist__double> not in the namelist?')
    end select

  end function namelist__double


!________________________________________________________________public__
!                                                                        !
  function namelist__integer(variable)                                   !
    character(len=*), intent(in) :: variable                             !
    integer :: namelist__integer                                         !
!________________________________________________________________________!
!
    if ( .not.Read_done ) then
       call ut__fatal('<namelist__integer> Read namelist file first.')
    end if

    select case (variable)
    case                 ('Avs_nskip')
       namelist__integer = Avs_nskip
    case                 ('Slicedata_nskip')
       namelist__integer = Slicedata_nskip
    case                 ('Total_nloop')
       namelist__integer = Total_nloop
    case default
       call ut__message('? arg = ', variable)
       call ut__fatal('<namelist__integer> not in the namelist?')
    end select

  end function namelist__integer


!________________________________________________________________public__
!                                                                        !
  function namelist__logical(variable)                                   !
    character(len=*), intent(in) :: variable                             !
    logical :: namelist__logical                                         !
!________________________________________________________________________!
!
    if ( .not.Read_done ) then
       call ut__fatal('<namelist__logical> Read namelist file first.')
    end if

    select case (variable)
    case                 ('Verbose')
       namelist__logical = Verbose
    case                 ('Debug')
       namelist__logical = Debug
    case default
       call ut__message('? arg = ', variable)
       call ut__fatal('<namelist__logical> not in the namelist?')
    end select

  end function namelist__logical


!________________________________________________________________public__
!                                                                        !
  subroutine namelist__read                                              !
!________________________________________________________________________!
!

    read(FILE_NAMELIST,nml=data00)
    read(FILE_NAMELIST,nml=data01)
    read(FILE_NAMELIST,nml=data02)
    read(FILE_NAMELIST,nml=data03)
    read(FILE_NAMELIST,nml=data04)

    write(FILE_STANDARD_OUT,nml=data00)
    write(FILE_STANDARD_OUT,nml=data01)
    write(FILE_STANDARD_OUT,nml=data02)
    write(FILE_STANDARD_OUT,nml=data03)
    write(FILE_STANDARD_OUT,nml=data04)

    Read_done = .true.

  end subroutine namelist__read


!________________________________________________________________public__
!                                                                        !
  function namelist__string(variable)                                    !
    character(len=*), intent(in) :: variable                             !
    character(len=TAG_STRING_LENGTH_MAX) :: namelist__string             !
!________________________________________________________________________!
!
    if ( .not.Read_done ) then
       call ut__fatal('<namelist__string> Read namelist file first.')
    end if

    select case         (variable)
    case                ('Avs_tag')
       namelist__string = Avs_tag
    case                ('Slicedata_tag')
       namelist__string = Slicedata_tag
    case default
       call ut__message('? arg = ', variable)
       call ut__fatal('<namelist__string> not in the namelist?')
    end select

  end function namelist__string

end module namelist
