!-----------------------------------------------------------------------------
! KINDANB: kinematic dynamo in a box
!-----------------------------------------------------------------------------
! rk4.f90
!              2006.09.20: Converted from Runge_kutta4.
!              2006.09.20: Created by Akira Kageyama. 
!-----------------------------------------------------------------------------

module rk4
!*****************************************************************************
! MODULE RK4                                4TH RUNGE-KUTTA INTEGRATION METHOD
!*****************************************************************************
  use constants
  use grid
  use field
  implicit none

  private
  public :: rk4__advance


contains

!_______________________________________________________________private__
!
  function induction(dt,eta,vel,mag)
    real(DP),               intent(in) :: dt, eta
    type(field__vector3d_), intent(in) :: vel
    type(field__vector3d_), intent(in) :: mag
    type(field__vector3d_)             :: induction
!________________________________________________________________________
!

    induction = dt*( .curl.(vel.x.mag) + eta*(.laplacian.mag) )
    
  end function induction


!_______________________________________________________________public___
!
  subroutine rk4__advance(t,dt,eta,vel,mag)
    real(DP), intent(inout)               :: t
    real(DP), intent(in)                  :: dt
    real(DP), intent(in)                  :: eta
    type(field__vector3d_), intent(in)    :: vel
    type(field__vector3d_), intent(inout) :: mag
!________________________________________________________________________
!
    real(DP), parameter :: ONE_SIXTH = 1.0_DP / 6.0_DP

    type(field__vector3d_) :: dmag01, dmag02, dmag03, dmag04
    type(field__vector3d_) :: mag_

    !--< step 1 >--!
    dmag01 = induction(dt, eta, vel, mag)

    !--< step 2 >--!
    mag_ = mag + dmag01*0.5_DP
    dmag02 = induction(dt, eta, vel, mag_)

    !--< step 3 >--!
    mag_ = mag + dmag02*0.5_DP
    dmag03 = induction(dt, eta, vel, mag_)

    !--< step 4 >--!
    mag_ = mag + dmag03
    dmag04 = induction(dt, eta, vel, mag_)
     
    !--< resuts >--!
    t = t + dt
    mag = mag + ONE_SIXTH*( dmag01+2.0_DP*dmag02+2.0_DP*dmag03+dmag04 )

  end subroutine rk4__advance

end module rk4
