!-----------------------------------------------------------------------------
! KINDANB: kinematic dynamo in a box
!                                           Akira Kageyama, kage@jamstec.go.jp
!                                              Earth Simulator Center, JAMSTEC
!-----------------------------------------------------------------------------
! slicedata.f90
!              2007.07.22: Created by Akira Kageyama.
!-----------------------------------------------------------------------------

module slicedata
!*****************************************************************************
! MODULE SLICEDATA                                    Generate 2-D Sliced Data
!*****************************************************************************
  use ut
  use field
  use namelist
  implicit none

  private
  public :: slicedata__initialize,  &
            slicedata__write

  ! - 2-D single precision real arrays.
  real(SP), dimension(:,:), allocatable :: Slice_vx  ! x-comp. of velocity
  real(SP), dimension(:,:), allocatable :: Slice_vy  ! y-comp.
  real(SP), dimension(:,:), allocatable :: Slice_vz  ! z-comp.
  real(SP), dimension(:,:), allocatable :: Slice_bx  ! x-comp. of mag. field
  real(SP), dimension(:,:), allocatable :: Slice_by  ! y-comp.
  real(SP), dimension(:,:), allocatable :: Slice_bz  ! z-comp.

contains


!===============
!    Private
!===============


!_______________________________________________________________private__
!                                                                        !
  subroutine make_single_precision_field(vel,mag)                        !
    type(field__vector3d_), intent(in) :: vel,mag                        !
!________________________________________________________________________!
!
    integer :: slice_k = NZ / 2

    Slice_vx = real(vel%x(:,:,slice_k),SP)
    Slice_vy = real(vel%y(:,:,slice_k),SP)
    Slice_vz = real(vel%z(:,:,slice_k),SP)

    Slice_bx = real(mag%x(:,:,slice_k),SP)
    Slice_by = real(mag%y(:,:,slice_k),SP)
    Slice_bz = real(mag%z(:,:,slice_k),SP)

  end subroutine make_single_precision_field


!==============
!    Public
!==============


!________________________________________________________________public__
!                                                                        !
  subroutine slicedata__initialize                                       !
!________________________________________________________________________!
!
    allocate(Slice_vx(NX,NY),   &
             Slice_vy(NX,NY),   &
             Slice_vz(NX,NY))

    allocate(Slice_bx(NX,NY),   &
             Slice_by(NX,NY),   &
             Slice_bz(NX,NY))

    open(FILE_SLICEDATA,                                &
         file=trim(namelist__string('Slicedata_tag')),  &
         form='unformatted')

  end subroutine slicedata__initialize


!________________________________________________________________public__
!                                                                        !
  subroutine slicedata__write(vel,mag,nloop,time)                        !
    type(field__vector3d_), intent(in) :: vel, mag                       !
    integer,                intent(in) :: nloop                          !
    real(DP),               intent(in) :: time                           !
!________________________________________________________________________!
!
    if ( namelist__integer('Slicedata_nskip') <= 0 ) return    
                                      ! Set zero or negative integer
                                      ! when you don't want to
                                      ! save any slice data.

    if (mod(nloop,namelist__integer('Slicedata_nskip')) /= 0) return

    call make_single_precision_field(vel,mag)


    write(FILE_SLICEDATA) nloop, real(time,SP),                 &
                          Slice_vx, Slice_vy, Slice_vz,         &
                          Slice_bx, Slice_by, Slice_bz

    call ut__message('#slice data saved at ', nloop, time)

  end subroutine slicedata__write

end module slicedata
