!-----------------------------------------------------------------------------
! KINDANB: kinematic dynamo in a box
!                                           Akira Kageyama, kage@jamstec.go.jp
!                                              Earth Simulator Center, JAMSTEC
!-----------------------------------------------------------------------------
! solenoidal.f90
!              2006.09.25: Use grid, no global.
!              2005.06.02: Created by Akira Kageyama.
!-----------------------------------------------------------------------------

module solenoidal
!*****************************************************************************
! MODULE SOLENOIDAL                         Makes Divergence-Free Vector Field
!*****************************************************************************
  ! - You may try various flow field for the kinematic dynamo through this.
  ! - This is also used to set the initial magnetic perturbation.
  ! - Try the Beltrami fields.
  !                                                 2005.06.02  Akira Kageyama
  !---------------------------------------------------------------------------
  use ut
  use grid
  use field
  implicit none

  public


contains


!==============
!    Public
!==============


!________________________________________________________________public__
!                                                                        !
  function solenoidal__abc_flow()                                        !
    type(field__vector3d_) :: solenoidal__abc_flow                       !
!________________________________________________________________________!
!
    integer  :: i, j, k
    real(DP) :: xx, yy, zz

    do k = 1 , NZ
       zz = grid__pos%z(k)
       do j = 1 , NY
          yy = grid__pos%y(j)
          do i = 1 , NX
             xx = grid__pos%x(i)
             solenoidal__abc_flow%x(i,j,k) = cos(yy) + sin(zz)
             solenoidal__abc_flow%y(i,j,k) = cos(zz) + sin(xx)
             solenoidal__abc_flow%z(i,j,k) = cos(xx) + sin(yy)
          end do
       end do
    end do

    solenoidal__abc_flow = .normalize.solenoidal__abc_flow

  end function solenoidal__abc_flow


!________________________________________________________________public__
!                                                                        !
  function solenoidal__sample01(l,m,n)                                   !
    integer,        intent(in) :: l, m, n                                !
    type(field__vector3d_)     :: solenoidal__sample01                   !
!________________________________________________________________________!
!
    solenoidal__sample01 = .curl.iTest(l,m,n)
    solenoidal__sample01 = .normalize.solenoidal__sample01

  contains

  !- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -!
    function iTest(l,m,n)                  ! A temporal vector potential.
      integer, intent(in) :: l, m, n
      type(field__vector3d_) :: iTest
  !- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -!
      integer  :: i, j, k
      real(DP) :: xx, yy, zz

      do k = 1 , NZ
         zz = grid__pos%z(k)
         do j = 1 , NY
            yy = grid__pos%y(j)
            do i = 1 , NX
               xx = grid__pos%x(i)
               iTest%x(i,j,k) = sin(l*xx)*sin(m*yy)*sin(n*zz)
               iTest%y(i,j,k) = sin(m*xx)*sin(m*zz)
               iTest%z(i,j,k) = sin(n*xx)
            end do
         end do
      end do

      call field__boundary_condition(iTest)

    end function iTest

  end function solenoidal__sample01


!________________________________________________________________public__
!                                                                        !
  function solenoidal__sample02()                                        !
    type(field__vector3d_) :: solenoidal__sample02                       !
!________________________________________________________________________!
!
    solenoidal__sample02%x = 0.0_DP
    solenoidal__sample02%y = 0.0_DP
    solenoidal__sample02%z = 1.0_DP

    call field__boundary_condition(solenoidal__sample02)

    solenoidal__sample02 = .normalize.solenoidal__sample02

  end function solenoidal__sample02

end module solenoidal
